@@ -1,30 +1,27 @@
-use crate::de::wbem_class_de::from_wbem_class_obj;
-use crate::result_enumerator::{IWbemClassWrapper, QueryResultEnumerator};
-use crate::BStr;
 use crate::{
-    connection::WMIConnection, de::meta::struct_name_and_fields, utils::check_hres, WMIError,
+    connection::WMIConnection,
+    de::meta::struct_name_and_fields,
+    de::wbem_class_de::from_wbem_class_obj,
+    result_enumerator::{IWbemClassWrapper, QueryResultEnumerator},
+    WMIError, WMIResult,
 };
 use log::trace;
 use serde::de;
-use std::collections::HashMap;
-use std::ptr;
-use std::ptr::NonNull;
-use winapi::um::wbemcli::IWbemClassObject;
-use winapi::{
-    shared::ntdef::NULL,
-    um::{
-        wbemcli::IEnumWbemClassObject,
-        wbemcli::{
-            WBEM_FLAG_FORWARD_ONLY, WBEM_FLAG_RETURN_IMMEDIATELY, WBEM_FLAG_RETURN_WBEM_COMPLETE,
-        },
-    },
+use std::{collections::HashMap, time::Duration};
+use windows::core::BSTR;
+use windows::Win32::System::Wmi::{
+    WBEM_FLAG_FORWARD_ONLY, WBEM_FLAG_RETURN_IMMEDIATELY, WBEM_FLAG_RETURN_WBEM_COMPLETE,
 };
 
+#[non_exhaustive]
 pub enum FilterValue {
     Bool(bool),
     Number(i64),
     Str(&'static str),
     String(String),
+    StrLike(&'static str),
+    StringLike(String),
+    IsA(&'static str),
 }
 
 impl From<String> for FilterValue {
@@ -51,7 +48,33 @@ impl From<bool> for FilterValue {
     }
 }
 
-/// Build an SQL query for the given filters, over the given type (using it's name and fields).
+impl FilterValue {
+    /// Create a [FilterValue::IsA] varinat form a given type
+    ///
+    /// ```edition2018
+    /// # use std::collections::HashMap;
+    /// # use wmi::FilterValue;
+    /// # use serde::Deserialize;
+    /// # fn main() -> wmi::WMIResult<()> {
+    /// #[derive(Deserialize)]
+    /// struct Win32_OperatingSystem {}
+    ///
+    /// let mut filters = HashMap::<String, FilterValue>::new();
+    /// filters.insert("TargetInstance".to_owned(), FilterValue::is_a::<Win32_OperatingSystem>()?);
+    /// # Ok(())
+    /// # }
+    /// ```
+    ///
+    pub fn is_a<'de, T>() -> WMIResult<Self>
+    where
+        T: serde::Deserialize<'de>,
+    {
+        let (name, _) = struct_name_and_fields::<T>()?;
+        Ok(Self::IsA(name))
+    }
+}
+
+/// Build an SQL query for the given filters, over the given type (using its name and fields).
 /// For example, for:
 ///
 /// # Examples
@@ -75,9 +98,76 @@ impl From<bool> for FilterValue {
 /// "SELECT Caption, Debug FROM Win32_OperatingSystem";
 /// ```
 ///
-pub fn build_query<'de, T>(
+pub fn build_query<'de, T>(filters: Option<&HashMap<String, FilterValue>>) -> WMIResult<String>
+where
+    T: de::Deserialize<'de>,
+{
+    let (name, fields, optional_where_clause) = get_query_segments::<T>(filters)?;
+
+    let query_text = format!(
+        "SELECT {} FROM {} {}",
+        fields.join(","),
+        name,
+        optional_where_clause
+    );
+
+    Ok(query_text)
+}
+
+/// Build an SQL query for an event notification subscription with the given filters and within polling time, over the given type (using its fields).
+/// For example, for:
+///
+/// # Examples
+///
+/// For a struct such as:
+///
+/// ```edition2018
+/// # use wmi::*;
+/// # use serde::Deserialize;
+/// #[derive(Deserialize, Debug)]
+/// #[serde(rename = "Win32_ProcessStartTrace")]
+/// #[serde(rename_all = "PascalCase")]
+/// struct ProcessStartTrace {
+///     process_id: u32,
+///     process_name: String,
+/// }
+/// ```
+///
+/// The resulting query with no filters and no within polling time will look like:
+/// ```
+/// "SELECT * FROM Win32_ProcessStartTrace";
+/// ```
+///
+/// Conversely, the resulting query with filters and with within polling time will look like:
+/// ```
+/// "SELECT * FROM Win32_ProcessStartTrace WITHIN 10 WHERE ProcessName = 'explorer.exe'";
+/// ```
+///
+pub fn build_notification_query<'de, T>(
     filters: Option<&HashMap<String, FilterValue>>,
-) -> Result<String, WMIError>
+    within: Option<Duration>,
+) -> WMIResult<String>
+where
+    T: de::Deserialize<'de>,
+{
+    let (name, _, optional_where_clause) = get_query_segments::<T>(filters)?;
+
+    let optional_within_caluse = match within {
+        Some(within) => format!("WITHIN {} ", within.as_secs_f64()),
+        None => String::new(),
+    };
+
+    let query_text = format!(
+        "SELECT * FROM {} {}{}",
+        name, optional_within_caluse, optional_where_clause
+    );
+
+    Ok(query_text)
+}
+
+fn get_query_segments<'de, T>(
+    filters: Option<&HashMap<String, FilterValue>>,
+) -> WMIResult<(&'static str, &'static [&'static str], String)>
 where
     T: de::Deserialize<'de>,
 {
@@ -102,7 +192,31 @@ where
                         }
                         FilterValue::Number(n) => format!("{}", n),
                         FilterValue::Str(s) => quote_and_escape_wql_str(s),
-                        FilterValue::String(s) => quote_and_escape_wql_str(&s),
+                        FilterValue::String(s) => quote_and_escape_wql_str(s),
+                        FilterValue::StrLike(s) => {
+                            conditions.push(format!(
+                                "{} LIKE {}",
+                                field,
+                                quote_and_escape_wql_str(s)
+                            ));
+                            continue;
+                        }
+                        FilterValue::StringLike(s) => {
+                            conditions.push(format!(
+                                "{} LIKE {}",
+                                field,
+                                quote_and_escape_wql_str(s)
+                            ));
+                            continue;
+                        }
+                        FilterValue::IsA(s) => {
+                            conditions.push(format!(
+                                "{} ISA {}",
+                                field,
+                                quote_and_escape_wql_str(s)
+                            ));
+                            continue;
+                        }
                     };
 
                     conditions.push(format!("{} = {}", field, value));
@@ -116,14 +230,7 @@ where
         }
     };
 
-    let query_text = format!(
-        "SELECT {} FROM {} {}",
-        fields.join(","),
-        name,
-        optional_where_clause
-    );
-
-    Ok(query_text)
+    Ok((name, fields.unwrap_or(&["*"]), optional_where_clause))
 }
 
 /// Quote/escape a string for WQL.
@@ -141,8 +248,9 @@ where
 /// # use wmi::query::quote_and_escape_wql_str;
 /// assert_eq!(quote_and_escape_wql_str(r#"C:\Path\With"In Name"#), r#""C:\\Path\\With\"In Name""#);
 /// ```
-pub fn quote_and_escape_wql_str(s: &str) -> String {
-    let mut o = String::with_capacity(s.as_bytes().len() + 2);
+pub fn quote_and_escape_wql_str(s: impl AsRef<str>) -> String {
+    let s = s.as_ref();
+    let mut o = String::with_capacity(s.len() + 2);
     o.push('"');
     for ch in s.chars() {
         match ch {
@@ -162,25 +270,22 @@ impl WMIConnection {
     pub fn exec_query_native_wrapper(
         &self,
         query: impl AsRef<str>,
-    ) -> Result<QueryResultEnumerator, WMIError> {
-        let query_language = BStr::from_str("WQL")?;
-        let query = BStr::from_str(query.as_ref())?;
+    ) -> WMIResult<QueryResultEnumerator> {
+        let query_language = BSTR::from("WQL");
+        let query = BSTR::from(query.as_ref());
 
-        let mut p_enumerator = NULL as *mut IEnumWbemClassObject;
+        let enumerator = unsafe {
+            self.svc.ExecQuery(
+                &query_language,
+                &query,
+                WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY,
+                &self.ctx.0,
+            )?
+        };
 
-        unsafe {
-            check_hres((*self.svc()).ExecQuery(
-                query_language.as_bstr(),
-                query.as_bstr(),
-                (WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY) as i32,
-                ptr::null_mut(),
-                &mut p_enumerator,
-            ))?;
-        }
+        trace!("Got enumerator {:?}", enumerator);
 
-        trace!("Got enumerator {:?}", p_enumerator);
-
-        Ok(unsafe { QueryResultEnumerator::new(self, p_enumerator) })
+        Ok(QueryResultEnumerator::new(self, enumerator))
     }
 
     /// Execute a free-text query and deserialize the results.
@@ -188,15 +293,15 @@ impl WMIConnection {
     /// but also with a generic map.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// # use std::collections::HashMap;
     /// # use wmi::*;
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// let results: Vec<HashMap<String, Variant>> = con.raw_query("SELECT Name FROM Win32_OperatingSystem")?;
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn raw_query<T>(&self, query: impl AsRef<str>) -> Result<Vec<T>, WMIError>
+    pub fn raw_query<T>(&self, query: impl AsRef<str>) -> WMIResult<Vec<T>>
     where
         T: de::DeserializeOwned,
     {
@@ -213,11 +318,11 @@ impl WMIConnection {
     /// Query all the objects of type T.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// use wmi::*;
     /// use serde::Deserialize;
     ///
-    /// let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// let con = WMIConnection::new(COMLibrary::new()?)?;
     ///
     /// #[derive(Deserialize, Debug)]
     /// struct Win32_Process {
@@ -228,22 +333,22 @@ impl WMIConnection {
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn query<T>(&self) -> Result<Vec<T>, WMIError>
+    pub fn query<T>(&self) -> WMIResult<Vec<T>>
     where
         T: de::DeserializeOwned,
     {
         let query_text = build_query::<T>(None)?;
 
-        self.raw_query(&query_text)
+        self.raw_query(query_text)
     }
 
     /// Query all the objects of type T, while filtering according to `filters`.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// # use std::collections::HashMap;
     /// # use wmi::*;
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// use serde::Deserialize;
     /// #[derive(Deserialize, Debug)]
     /// struct Win32_Process {
@@ -260,16 +365,13 @@ impl WMIConnection {
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn filtered_query<T>(
-        &self,
-        filters: &HashMap<String, FilterValue>,
-    ) -> Result<Vec<T>, WMIError>
+    pub fn filtered_query<T>(&self, filters: &HashMap<String, FilterValue>) -> WMIResult<Vec<T>>
     where
         T: de::DeserializeOwned,
     {
-        let query_text = build_query::<T>(Some(&filters))?;
+        let query_text = build_query::<T>(Some(filters))?;
 
-        self.raw_query(&query_text)
+        self.raw_query(query_text)
     }
 
     /// Get a single object of type T.
@@ -277,8 +379,8 @@ impl WMIConnection {
     /// If more than one object is found, all but the first are ignored.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # fn main() -> wmi::WMIResult<()> {
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// # use wmi::*;
     /// use serde::Deserialize;
     /// #[derive(Deserialize)]
@@ -290,26 +392,23 @@ impl WMIConnection {
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn get<T>(&self) -> Result<T, WMIError>
+    pub fn get<T>(&self) -> WMIResult<T>
     where
         T: de::DeserializeOwned,
     {
         let results = self.query()?;
 
-        results
-            .into_iter()
-            .next()
-            .ok_or_else(|| WMIError::ResultEmpty)
+        results.into_iter().next().ok_or(WMIError::ResultEmpty)
     }
 
     /// Get a WMI object by path, and return a wrapper around a WMI pointer.
     /// It's better to use the `get_by_path` method, since this function is more low level.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// # use wmi::*;
     /// # use serde::Deserialize;
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// let raw_os = con.get_raw_by_path(r#"\\.\root\cimv2:Win32_OperatingSystem=@"#)?;
     /// assert_eq!(raw_os.class()?, "Win32_OperatingSystem");
     ///
@@ -323,27 +422,24 @@ impl WMIConnection {
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn get_raw_by_path(
-        &self,
-        object_path: impl AsRef<str>,
-    ) -> Result<IWbemClassWrapper, WMIError> {
-        let object_path = BStr::from_str(object_path.as_ref())?;
+    pub fn get_raw_by_path(&self, object_path: impl AsRef<str>) -> WMIResult<IWbemClassWrapper> {
+        let object_path = BSTR::from(object_path.as_ref());
 
-        let mut pcls_obj = NULL as *mut IWbemClassObject;
+        let mut pcls_obj = None;
 
         unsafe {
-            check_hres((*self.svc()).GetObject(
-                object_path.as_bstr(),
-                WBEM_FLAG_RETURN_WBEM_COMPLETE as i32,
-                ptr::null_mut(),
-                &mut pcls_obj,
-                ptr::null_mut(),
-            ))?;
+            self.svc.GetObject(
+                &object_path,
+                WBEM_FLAG_RETURN_WBEM_COMPLETE,
+                &self.ctx.0,
+                Some(&mut pcls_obj),
+                None,
+            )?;
         }
 
-        let pcls_ptr = NonNull::new(pcls_obj).ok_or(WMIError::NullPointerResult)?;
+        let pcls_ptr = pcls_obj.ok_or(WMIError::NullPointerResult)?;
 
-        let pcls_wrapper = unsafe { IWbemClassWrapper::new(pcls_ptr) };
+        let pcls_wrapper = IWbemClassWrapper::new(pcls_ptr);
 
         Ok(pcls_wrapper)
     }
@@ -352,10 +448,10 @@ impl WMIConnection {
     /// This is useful when the type of the object at the path in known at compile time.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// # use wmi::*;
     /// # use serde::Deserialize;
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// #[derive(Deserialize)]
     /// struct Win32_OperatingSystem {
     ///     Name: String,
@@ -370,11 +466,11 @@ impl WMIConnection {
     /// or if the object is only of a few possible types, deserialize it to an enum:
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// # use std::collections::HashMap;
     /// # use wmi::*;
     /// # use serde::Deserialize;
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     ///
     /// #[derive(Deserialize, Debug, PartialEq)]
     /// struct Win32_Group {
@@ -420,7 +516,7 @@ impl WMIConnection {
     ///     User(Win32_UserAccount),
     ///     #[serde(rename = "Win32_Group")]
     ///     Group(Win32_Group),
-    /// };
+    /// }
     ///
     /// for account in accounts_in_group {
     ///     let user: User = con.get_by_path(&account.__Path)?;
@@ -429,25 +525,25 @@ impl WMIConnection {
     /// #   Ok(())
     /// # }
     /// ```
-    pub fn get_by_path<T>(&self, object_path: &str) -> Result<T, WMIError>
+    pub fn get_by_path<T>(&self, object_path: &str) -> WMIResult<T>
     where
         T: de::DeserializeOwned,
     {
         let wbem_class_obj = self.get_raw_by_path(object_path)?;
 
-        from_wbem_class_obj(&wbem_class_obj)
+        from_wbem_class_obj(wbem_class_obj)
     }
 
     /// Query all the associators of type T of the given object.
     /// The `object_path` argument can be provided by querying an object wih it's `__Path` property.
-    /// `AssocClass` must be have the name as the conneting association class between the original object and the results.
+    /// `AssocClass` must be have the name as the connecting association class between the original object and the results.
     /// See <https://docs.microsoft.com/en-us/windows/desktop/cimwin32prov/win32-diskdrivetodiskpartition> for example.
     ///
     /// ```edition2018
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> wmi::WMIResult<()> {
     /// # use wmi::*;
     /// # use serde::Deserialize;
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     ///
     /// #[derive(Deserialize, Debug)]
     /// struct Win32_DiskDrive {
@@ -473,7 +569,7 @@ impl WMIConnection {
     pub fn associators<ResultClass, AssocClass>(
         &self,
         object_path: &str,
-    ) -> Result<Vec<ResultClass>, WMIError>
+    ) -> WMIResult<Vec<ResultClass>>
     where
         ResultClass: de::DeserializeOwned,
         AssocClass: de::DeserializeOwned,
@@ -490,22 +586,22 @@ impl WMIConnection {
             class_name = class_name
         );
 
-        self.raw_query(&query)
+        self.raw_query(query)
     }
 }
 
 #[allow(non_snake_case)]
 #[allow(non_camel_case_types)]
+#[allow(dead_code)]
 #[cfg(test)]
 mod tests {
     use super::*;
     use serde::Deserialize;
     use std::collections::HashMap;
+    use windows::Win32::System::Wmi::WBEM_E_INVALID_QUERY;
 
     use crate::tests::fixtures::*;
     use crate::{Variant, WMIError};
-    use winapi::shared::ntdef::HRESULT;
-    use winapi::um::wbemcli::WBEM_E_INVALID_QUERY;
 
     #[test]
     fn it_works() {
@@ -523,7 +619,12 @@ mod tests {
 
             assert_eq!(props.len(), 64);
             assert_eq!(props[..2], ["BootDevice", "BuildNumber"]);
-            assert_eq!(props[props.len() - 2..], ["Version", "WindowsDirectory"])
+            assert_eq!(props[props.len() - 2..], ["Version", "WindowsDirectory"]);
+
+            let result = serde_json::to_string_pretty(&w);
+
+            assert!(result.is_ok());
+            assert!(result.unwrap().len() > 2)
         }
     }
 
@@ -552,7 +653,7 @@ mod tests {
                 Ok(_) => assert!(false),
                 Err(wmi_err) => match wmi_err {
                     WMIError::HResultError { hres } => {
-                        assert_eq!(hres, WBEM_E_INVALID_QUERY as HRESULT);
+                        assert_eq!(hres, WBEM_E_INVALID_QUERY.0);
                     }
                     _ => assert!(false),
                 },
@@ -588,13 +689,73 @@ mod tests {
         assert!(results_as_json.starts_with(r#"[{"Name":"Microsoft Windows"#));
     }
 
+    #[test]
+    fn it_builds_correct_query_for_newtype_struct() {
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem(pub HashMap<String, Variant>);
+
+        let query = build_query::<Win32_OperatingSystem>(None).unwrap();
+        let select_part = r#"SELECT * FROM Win32_OperatingSystem "#.to_owned();
+
+        assert_eq!(query, select_part);
+    }
+
+    #[test]
+    fn it_can_query_a_newtype_struct() {
+        let wmi_con = wmi_con();
+
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem(pub HashMap<String, Variant>);
+
+        let results = wmi_con.query::<Win32_OperatingSystem>().unwrap();
+
+        for os in results {
+            match os.0.get("Caption").unwrap() {
+                Variant::String(s) => assert!(s.starts_with("Microsoft Windows")),
+                _ => assert!(false),
+            }
+        }
+    }
+
+    #[test]
+    fn con_query_flatten() {
+        // Due to serde#1346, it's not possible to use `query` with a struct that has a `flatten` field,
+        // so we need to either use `raw_query` or a newtype struct.
+
+        let wmi_con = wmi_con();
+
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem {
+            Caption: String,
+            Name: String,
+
+            #[serde(flatten)]
+            extra: HashMap<String, Variant>,
+        }
+
+        let system: Vec<Win32_OperatingSystem> = wmi_con
+            .raw_query("SELECT * FROM Win32_OperatingSystem")
+            .unwrap();
+        let system = system.into_iter().next().unwrap();
+        assert_ne!(system.Name, "");
+        assert!(system.extra.contains_key("BuildNumber"));
+
+        #[derive(Deserialize, Debug)]
+        #[serde(rename = "Win32_OperatingSystem")]
+        struct Win32_OperatingSystemWrapper(pub Win32_OperatingSystem);
+        let system = wmi_con.query::<Win32_OperatingSystemWrapper>().unwrap();
+        let system = system.into_iter().next().unwrap();
+
+        assert_ne!(system.0.Name, "");
+        assert!(system.0.extra.contains_key("BuildNumber"));
+    }
+
     #[test]
     fn it_fails_gracefully_when_querying_a_struct() {
         let wmi_con = wmi_con();
 
         #[derive(Deserialize, Debug)]
         struct Win32_OperatingSystem {
-            #[allow(dead_code)]
             NoSuchField: String,
         }
 
@@ -607,7 +768,6 @@ mod tests {
     fn it_builds_correct_query_without_filters() {
         #[derive(Deserialize, Debug)]
         struct Win32_OperatingSystem {
-            #[allow(dead_code)]
             Caption: String,
         }
 
@@ -617,32 +777,88 @@ mod tests {
         assert_eq!(query, select_part);
     }
 
+    #[test]
+    fn it_builds_correct_notification_query_without_filters() {
+        #[derive(Deserialize, Debug)]
+        struct Win32_ProcessStartTrace {
+            Caption: String,
+        }
+
+        let query = build_notification_query::<Win32_ProcessStartTrace>(None, None).unwrap();
+        let select_part = r#"SELECT * FROM Win32_ProcessStartTrace "#.to_owned();
+
+        assert_eq!(query, select_part);
+    }
+
     #[test]
     fn it_builds_correct_query() {
         #[derive(Deserialize, Debug)]
         struct Win32_OperatingSystem {
-            #[allow(dead_code)]
             Caption: String,
         }
 
         let mut filters = HashMap::new();
 
-        filters.insert("C1".to_string(), FilterValue::Str("a"));
-        filters.insert("C2".to_string(), FilterValue::String("b".to_string()));
-        filters.insert("C3".to_string(), FilterValue::Number(42));
-        filters.insert("C4".to_string(), FilterValue::Bool(false));
+        filters.insert("C1".to_owned(), FilterValue::Str("a"));
+        filters.insert("C2".to_owned(), FilterValue::String("b".to_owned()));
+        filters.insert("C3".to_owned(), FilterValue::Number(42));
+        filters.insert("C4".to_owned(), FilterValue::Bool(false));
         filters.insert(
-            "C5".to_string(),
-            FilterValue::String(r#"with " and \ chars"#.to_string()),
+            "C5".to_owned(),
+            FilterValue::String(r#"with " and \ chars"#.to_owned()),
         );
+        filters.insert("C6".to_owned(), FilterValue::IsA("Class"));
+        filters.insert(
+            "C7".to_owned(),
+            FilterValue::is_a::<Win32_OperatingSystem>().unwrap(),
+        );
+        filters.insert("C8".to_owned(), FilterValue::StrLike("c"));
+        filters.insert("C9".to_owned(), FilterValue::StringLike("d".to_owned()));
 
         let query = build_query::<Win32_OperatingSystem>(Some(&filters)).unwrap();
         let select_part = r#"SELECT Caption FROM Win32_OperatingSystem "#.to_owned();
-        let where_part = r#"WHERE C1 = "a" AND C2 = "b" AND C3 = 42 AND C4 = false AND C5 = "with \" and \\ chars""#;
+        let where_part = r#"WHERE C1 = "a" AND C2 = "b" AND C3 = 42 AND C4 = false AND C5 = "with \" and \\ chars" AND C6 ISA "Class" AND C7 ISA "Win32_OperatingSystem" AND C8 LIKE "c" AND C9 LIKE "d""#;
 
         assert_eq!(query, select_part + where_part);
     }
 
+    #[test]
+    fn it_builds_correct_notification_query() {
+        #[derive(Deserialize, Debug)]
+        struct Win32_ProcessStartTrace {
+            Caption: String,
+        }
+
+        let mut filters = HashMap::new();
+
+        filters.insert("C1".to_owned(), FilterValue::Str("a"));
+        filters.insert("C2".to_owned(), FilterValue::String("b".to_owned()));
+        filters.insert("C3".to_owned(), FilterValue::Number(42));
+        filters.insert("C4".to_owned(), FilterValue::Bool(false));
+        filters.insert(
+            "C5".to_owned(),
+            FilterValue::String(r#"with " and \ chars"#.to_owned()),
+        );
+        filters.insert("C6".to_owned(), FilterValue::IsA("Class"));
+        filters.insert(
+            "C7".to_owned(),
+            FilterValue::is_a::<Win32_ProcessStartTrace>().unwrap(),
+        );
+        filters.insert("C8".to_owned(), FilterValue::StrLike("c"));
+        filters.insert("C9".to_owned(), FilterValue::StringLike("d".to_owned()));
+
+        let query = build_notification_query::<Win32_ProcessStartTrace>(
+            Some(&filters),
+            Some(Duration::from_secs_f64(10.5)),
+        )
+        .unwrap();
+        let select_part = r#"SELECT * FROM Win32_ProcessStartTrace "#.to_owned();
+        let within_part = r#"WITHIN 10.5 "#;
+        let where_part = r#"WHERE C1 = "a" AND C2 = "b" AND C3 = 42 AND C4 = false AND C5 = "with \" and \\ chars" AND C6 ISA "Class" AND C7 ISA "Win32_ProcessStartTrace" AND C8 LIKE "c" AND C9 LIKE "d""#;
+
+        assert_eq!(query, select_part + within_part + where_part);
+    }
+
     #[test]
     fn it_can_filter() {
         let wmi_con = wmi_con();
@@ -718,7 +934,7 @@ mod tests {
 
             for res in results {
                 match res.get("Antecedent") {
-                    Some(Variant::String(s)) => assert!(s != ""),
+                    Some(Variant::String(s)) => assert_ne!(s, ""),
                     _ => assert!(false),
                 }
             }
@@ -729,12 +945,12 @@ mod tests {
 
         for res in results {
             match res.get("GroupComponent") {
-                Some(Variant::String(s)) => assert!(s != ""),
+                Some(Variant::String(s)) => assert_ne!(s, ""),
                 _ => assert!(false),
             }
 
             match res.get("PartComponent") {
-                Some(Variant::String(s)) => assert!(s != ""),
+                Some(Variant::String(s)) => assert_ne!(s, ""),
                 _ => assert!(false),
             }
         }
@@ -770,7 +986,6 @@ mod tests {
         #[derive(Deserialize, Debug)]
         struct Win32_DiskDrive {
             __Path: String,
-            #[allow(dead_code)]
             Caption: String,
         }
 
@@ -791,7 +1006,10 @@ mod tests {
         assert!(results.len() >= 1);
 
         for part in results {
-            assert!(part.Caption.contains("Partition #"));
+            // We want to check that the output is in the format "Disk #1, Partition #1".
+            // However, it is localised so we simply check if there are two or more '#'.
+            // This means there are at least two sublevels in the hierarchy being enumerated.
+            assert!(part.Caption.chars().filter(|x| *x == '#').count() >= 2);
         }
     }
 
@@ -799,11 +1017,11 @@ mod tests {
     fn it_can_query_correct_variant_types() {
         let wmi_con = wmi_con();
         let mut results: Vec<HashMap<String, Variant>> = wmi_con
-            .raw_query("SELECT SystemStabilityIndex FROM Win32_ReliabilityStabilityMetrics")
+            .raw_query("SELECT CPUScore FROM Win32_WinSAT")
             .unwrap();
 
         match results.pop().unwrap().values().next() {
-            Some(&Variant::R8(_v)) => assert!(true),
+            Some(&Variant::R4(_v)) => assert!(true),
             _ => assert!(false),
         }
 
@@ -864,14 +1082,6 @@ mod tests {
     fn it_can_query_floats() {
         let wmi_con = wmi_con();
 
-        #[derive(Deserialize, Debug)]
-        struct Win32_ReliabilityStabilityMetrics {
-            SystemStabilityIndex: f64,
-        }
-
-        let metric = wmi_con.get::<Win32_ReliabilityStabilityMetrics>().unwrap();
-        assert!(metric.SystemStabilityIndex >= 0.0);
-
         #[derive(Deserialize, Debug)]
         struct Win32_WinSAT {
             CPUScore: f32,
